@@ -3,8 +3,8 @@
 ;; Copyright (C) 2013-2022  Free Software Foundation, Inc.
 
 ;; Author: Leo Liu <sdl.web@gmail.com>
-;; Package-Version: 20260105.825
-;; Package-Revision: 3c0fbada8ca5
+;; Package-Version: 20260121.752
+;; Package-Revision: 98cbae5d8c37
 ;; Package-Type: simple
 ;; Package-Requires: ((emacs "25") (cl-lib "0.5"))
 ;; Keywords: killing, convenience
@@ -75,11 +75,29 @@ string) for appending current selection to previous kill."
   :type '(repeat symbol)
   :group 'killing)
 
+(defcustom easy-kill-exit-hook nil
+  "A list of functions to be called when `easy-kill' exits.
+
+At the moment they are run, the current candidate is still accessible
+and `this-command' has the command that is to put an end to the
+easy-kill mode."
+  :type '(repeat function)
+  :group 'killing)
+
 (defcustom easy-mark-try-things '(url email uuid sexp)
   "A list of things for `easy-mark' to try."
   :type '(repeat symbol)
   :group 'killing)
 
+(defcustom easy-mark-exit-hook nil
+  "A list of functions to be called when `easy-mark' exits.
+
+At the moment they are run, the current candidate is still accessible
+and `this-command' has the command that is to put an end to the
+easy-mark mode."
+  :type '(repeat function)
+  :group 'killing)
+
 (defcustom easy-dup-try-things '(line)
   "A list of things for `easy-dup-before' and `easy-dup-after' to try."
   :type '(repeat symbol)
@@ -573,8 +591,11 @@ checked."
                           (command-remapping cmd nil (list map)))))
                (ignore
                 (easy-kill-destroy-candidate)
-                (unless (or (easy-kill-get mark) (easy-kill-exit-p this-command))
-                  (easy-kill-save-candidate))))
+                (if (easy-kill-get mark)
+                    (run-hooks 'easy-mark-exit-hook)
+                  (or (easy-kill-exit-p this-command)
+                      (easy-kill-save-candidate))
+                  (run-hooks 'easy-kill-exit-hook))))
          (error (message "%s:%s" this-command (error-message-string err))
                 nil))))))
 
@@ -626,10 +647,12 @@ Temporally activate additional key bindings as follows:
 ;;;###autoload
 (defun easy-dup (&optional n before)
   "Insert a copy of the current selection after it, or before it if BEFORE.
-When not in easy-kill/easy-mark, use the active region if available, or
-enter easy-mark using `easy-dup-try-things' to select something to
-duplicate.  `rectangle-mark-mode' is also supported.  N specifies the
-number of copies to insert."
+When not in easy-kill/easy-mark, enter `easy-mark' mode using the active
+region if available, or using `easy-dup-try-things' to select something
+to duplicate.  When in `rectangle-mark-mode', duplicate the rectangle
+region and keep the mode active without entering `easy-mark'.
+
+N specifies the number of copies to insert."
   (interactive "*p")
   (or
    (pcase (if easy-kill-candidate (easy-kill-get bounds) '(nil . nil))
@@ -642,13 +665,14 @@ number of copies to insert."
        ((use-region-p)
         (let* ((beg (region-beginning))
                (end (region-end))
-               (text (buffer-substring beg end)))
-          (save-excursion
-            (goto-char (if before beg end))
-            (if before
-                (dotimes (_ (or n 1)) (insert-before-markers text))
-              (dotimes (_ (or n 1)) (insert text)))))
-        t)
+               (easy-kill-try-things nil))
+          (easy-kill-init-candidate 1 'mark)
+          (setf (easy-kill-get thing) 'word)
+          (setf (easy-kill-get bounds) (cons beg end))
+          (setf (easy-kill-get mark) 'start)
+          (easy-kill-indicate-origin)
+          (easy-kill-activate-keymap)
+          nil))
        (t (let ((easy-mark-try-things easy-dup-try-things))
             (easy-mark 1)
             nil)))))
@@ -671,10 +695,12 @@ number of copies to insert."
 ;;;###autoload
 (defun easy-dup-before (&optional n)
   "Insert a copy of the current selection before it.
-When not in easy-kill/easy-mark, use the active region if available, or
-enter easy-mark using `easy-dup-try-things' to select something to
-duplicate.  `rectangle-mark-mode' is also supported.  N specifies the
-number of copies to insert."
+When not in easy-kill/easy-mark, enter `easy-mark' mode using the active
+region if available, or using `easy-dup-try-things' to select something
+to duplicate.  When in `rectangle-mark-mode', duplicate the rectangle
+region and keep the mode active without entering `easy-mark'.
+
+N specifies the number of copies to insert."
   (interactive "*p")
   (easy-dup n t))
 
